@@ -1,13 +1,33 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import print_function
 
 from newrelic.admin import command, usage
 
+
 def _run_validation_test():
     import time
 
-    from newrelic.agent import (background_task, error_trace,
-            external_trace, function_trace, wsgi_application,
-            add_custom_parameter, record_exception, application)
+    from newrelic.api.application import application_instance as application
+    from newrelic.api.background_task import background_task
+    from newrelic.api.error_trace import error_trace
+    from newrelic.api.external_trace import external_trace
+    from newrelic.api.function_trace import function_trace
+    from newrelic.api.transaction import add_custom_parameter
+    from newrelic.api.time_trace import record_exception
+    from newrelic.api.wsgi_application import wsgi_application
 
     @external_trace(library='test',
             url='http://localhost/test', method='GET')
@@ -29,14 +49,24 @@ def _run_validation_test():
     def _function3():
         add_custom_parameter('txn-key-1', 1)
 
-        try:
-            raise NotImplementedError(
-                    'This is a test error and can be ignored.')
-        except:
-            record_exception(params={'err-key-2': 2, 'err-key-3': 3.0})
+        _function4()
 
         raise RuntimeError('This is a test error and can be ignored.')
 
+    @function_trace()
+    def _function4(params=None, application=None):
+        try:
+            _function5()
+        except:
+            record_exception(params=(params or {
+                    'err-key-2': 2, 'err-key-3': 3.0}),
+                    application=application)
+
+    @function_trace()
+    def _function5():
+        raise NotImplementedError(
+                'This is a test error and can be ignored.')
+
     @wsgi_application()
     def _wsgi_application(environ, start_response):
         status = '200 OK'
@@ -85,10 +115,8 @@ def _run_validation_test():
 
     _background_task()
 
-    try:
-        raise NotImplementedError('This is a test error and can be ignored.')
-    except Exception:
-        record_exception(params={'err-key-4': 4}, application=application())
+    _function4(params={'err-key-4': 4, 'err-key-5': 5.0},
+            application=application())
 
 _user_message = """
 Running Python agent test.
@@ -123,8 +151,9 @@ def validate_config(args):
         usage('validate-config')
         sys.exit(1)
 
-    from newrelic.agent import (global_settings, initialize,
-            register_application)
+    from newrelic.api.application import register_application
+    from newrelic.config import initialize
+    from newrelic.core.config import global_settings
 
     if len(args) >= 2:
         log_file = args[1]
@@ -175,8 +204,6 @@ def validate_config(args):
     _logger.debug('Proxy port is %r.', _settings.proxy_port)
     _logger.debug('Proxy user is %r.', _settings.proxy_user)
 
-    _logger.debug('SSL enabled is %r.', _settings.ssl)
-
     _logger.debug('License key is %r.', _settings.license_key)
 
     _timeout = 30.0
