@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """Database utilities consists of routines for obfuscating SQL, retrieving
 explain plans for SQL etc.
 
@@ -5,10 +19,11 @@ explain plans for SQL etc.
 
 import logging
 import re
-import time
 import weakref
 
-from newrelic.core.internal_metrics import (internal_trace, internal_metric)
+import newrelic.packages.six as six
+
+from newrelic.core.internal_metrics import internal_metric
 from newrelic.core.config import global_settings
 
 _logger = logging.getLogger(__name__)
@@ -25,13 +40,30 @@ _logger = logging.getLogger(__name__)
 # no backslash. Uses two successive instances of quote character in
 # middle of the string to indicate one embedded quote.
 
-_single_quotes_p = "'(?:[^']|'')*'"
-_double_quotes_p = '"(?:[^"]|"")*"'
+_single_quotes_p = r"'(?:[^']|'')*?(?:\\'.*|'(?!'))"
+_double_quotes_p = r'"(?:[^"]|"")*?(?:\\".*|"(?!"))'
+_dollar_quotes_p = r'(\$(?!\d)[^$]*?\$).*?(?:\1|$)'
+_oracle_quotes_p = (r"q'\[.*?(?:\]'|$)|q'\{.*?(?:\}'|$)|"
+        r"q'\<.*?(?:\>'|$)|q'\(.*?(?:\)'|$)")
 _any_quotes_p = _single_quotes_p + '|' + _double_quotes_p
+_single_dollar_p = _single_quotes_p + '|' + _dollar_quotes_p
+_single_oracle_p = _single_quotes_p + '|' + _oracle_quotes_p
 
 _single_quotes_re = re.compile(_single_quotes_p)
-_double_quotes_re = re.compile(_double_quotes_p)
 _any_quotes_re = re.compile(_any_quotes_p)
+_single_dollar_re = re.compile(_single_dollar_p)
+_single_oracle_re = re.compile(_single_oracle_p)
+
+# Cleanup regexes. Presence of a quote will indicate that the now obfuscated
+# sql was actually malformed.
+
+_single_quotes_cleanup_p = r"'"
+_any_quotes_cleanup_p = r'\'|"'
+_single_dollar_cleanup_p = r"'|\$(?!\?)"
+
+_any_quotes_cleanup_re = re.compile(_any_quotes_cleanup_p)
+_single_quotes_cleanup_re = re.compile(_single_quotes_cleanup_p)
+_single_dollar_cleanup_re = re.compile(_single_dollar_cleanup_p)
 
 # See http://www.regular-expressions.info/examplesprogrammer.html.
 #
@@ -45,30 +77,43 @@ _any_quotes_re = re.compile(_any_quotes_p)
 # follows on from a ':'. This is because ':1' can be used as positional
 # parameter with database adapters where 'paramstyle' is 'numeric'.
 
-_int_re = re.compile(r'(?<!:)\b\d+\b')
+_uuid_p = r'\{?(?:[0-9a-f]\-?){32}\}?'
+_int_p = r'(?<!:)-?\b(?:[0-9]+\.)?[0-9]+(e[+-]?[0-9]+)?'
+_hex_p = r'0x[0-9a-f]+'
+_bool_p = r'\b(?:true|false|null)\b'
+
+# Join all literals into one compiled regular expression. Longest expressions
+# first to avoid the situation of partial matches on shorter expressions. UUIDs
+# might be an example.
+
+_all_literals_p = '(' + ')|('.join([_uuid_p, _hex_p, _int_p, _bool_p]) + ')'
+_all_literals_re = re.compile(_all_literals_p, re.IGNORECASE)
 
 _quotes_table = {
-    'single': _single_quotes_re,
-    'double': _double_quotes_re,
-    'single+double': _any_quotes_re,
+    'single': (_single_quotes_re, _single_quotes_cleanup_re),
+    'single+double': (_any_quotes_re, _any_quotes_cleanup_re),
+    'single+dollar': (_single_dollar_re, _single_dollar_cleanup_re),
+    'single+oracle': (_single_oracle_re, _single_quotes_cleanup_re),
 }
 
-_quotes_default = _single_quotes_re
 
-@internal_trace('Supportability/DatabaseUtils/Calls/obfuscate_sql')
 def _obfuscate_sql(sql, database):
-    quotes_re = _quotes_table.get(database.quoting_style, _single_quotes_re)
+    quotes_re, quotes_cleanup_re = _quotes_table.get(database.quoting_style,
+            (_single_quotes_re, _single_quotes_cleanup_re))
 
     # Substitute quoted strings first.
 
     sql = quotes_re.sub('?', sql)
 
-    # Replace straight integer values. This will pick up
-    # integers by themselves but also as part of floating point
-    # numbers. Because of word boundary checks in pattern will
-    # not match numbers within identifier names.
+    # Replace all other sensitive fields
 
-    sql = _int_re.sub('?', sql)
+    sql = _all_literals_re.sub('?', sql)
+
+    # Determine if the obfuscated query was malformed by searching for
+    # remaining quote characters
+
+    if quotes_cleanup_re.search(sql):
+        sql = '?'
 
     return sql
 
@@ -90,6 +135,7 @@ def _obfuscate_sql(sql, database):
 # ':\w+'. This can match ':1name', which is not strictly correct, but
 # then it likely isn't valid in SQL anyway for that param style.
 
+
 _normalize_params_1_p = r'%\([^)]*\)s'
 _normalize_params_1_re = re.compile(_normalize_params_1_p)
 _normalize_params_2_p = r'%s'
@@ -107,7 +153,7 @@ _normalize_whitespace_2_re = re.compile(_normalize_whitespace_2_p)
 _normalize_whitespace_3_p = r'(?<!\w)\s+'
 _normalize_whitespace_3_re = re.compile(_normalize_whitespace_3_p)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/normalize_sql')
+
 def _normalize_sql(sql):
     # Note we that do this as a series of regular expressions as
     # using '|' in regular expressions is more expensive.
@@ -150,15 +196,21 @@ def _normalize_sql(sql):
 # TODO This isn't always going to do what we require, but it is only used
 # now for cases below which never get invoked, so is okay for now.
 
-_identifier_re = re.compile('[\',"`\[\]\(\)]*')
+
+_identifier_re = re.compile(r'[\',"`\[\]\(\)]*')
+
 
 def _extract_identifier(token):
     return _identifier_re.sub('', token).strip().lower()
 
+
 # Helper function for removing C style comments embedded in SQL statements.
 
-_uncomment_sql_p = r'/\*.*?\*/'
-_uncomment_sql_re = re.compile(_uncomment_sql_p, re.DOTALL)
+_uncomment_sql_p = r'(?:#|--).*?(?=\r|\n|$)'
+_uncomment_sql_q = r'\/\*(?:[^\/]|\/[^*])*?(?:\*\/|\/\*.*)'
+_uncomment_sql_x = r'(%s)|(%s)' % (_uncomment_sql_p, _uncomment_sql_q)
+_uncomment_sql_re = re.compile(_uncomment_sql_x, re.DOTALL)
+
 
 def _uncomment_sql(sql):
     return _uncomment_sql_re.sub('', sql)
@@ -201,7 +253,7 @@ def _uncomment_sql(sql):
 # avoid trying to use \w pattern even if LOCALE flag is used.
 #
 # On top of the character set issues and quoting, different types of
-# bracketing such as () and [] can also be used around names.
+# bracketing such as (), {}, and [] can also be used around names.
 #
 # Because of the difficulty in handling locales especially, what we do
 # instead is try and match based on whatever occurs between the
@@ -213,132 +265,156 @@ def _uncomment_sql(sql):
 # You cannot mix different quoting schemes as then the regex gets
 # even more messy.
 
+
 def _parse_default(sql, regex):
     match = regex.search(sql)
     return match and _extract_identifier(match.group(1)) or ''
 
+
 _parse_identifier_1_p = r'"((?:[^"]|"")+)"(?:\."((?:[^"]|"")+)")?'
 _parse_identifier_2_p = r"'((?:[^']|'')+)'(?:\.'((?:[^']|'')+)')?"
 _parse_identifier_3_p = r'`((?:[^`]|``)+)`(?:\.`((?:[^`]|``)+)`)?'
 _parse_identifier_4_p = r'\[\s*(\S+)\s*\]'
 _parse_identifier_5_p = r'\(\s*(\S+)\s*\)'
-_parse_identifier_6_p = r'([^\s\(\)\[\],]+)'
+_parse_identifier_6_p = r'\{\s*(\S+)\s*\}'
+_parse_identifier_7_p = r'([^\s\(\)\[\],]+)'
 
 _parse_identifier_p = ''.join(('(', _parse_identifier_1_p, '|',
         _parse_identifier_2_p, '|', _parse_identifier_3_p, '|',
         _parse_identifier_4_p, '|', _parse_identifier_5_p, '|',
-        _parse_identifier_6_p, ')'))
+        _parse_identifier_6_p, '|', _parse_identifier_7_p, ')'))
 
-_parse_from_p = '\s+FROM\s+' + _parse_identifier_p
+_parse_from_p = r'\s+FROM\s+' + _parse_identifier_p
 _parse_from_re = re.compile(_parse_from_p, re.IGNORECASE)
 
+
 def _join_identifier(m):
     return m and '.'.join([s for s in m.groups()[1:] if s]).lower() or ''
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_select')
+
 def _parse_select(sql):
     return _join_identifier(_parse_from_re.search(sql))
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_delete')
+
 def _parse_delete(sql):
     return _join_identifier(_parse_from_re.search(sql))
 
-_parse_into_p = '\s+INTO\s+' + _parse_identifier_p
+
+_parse_into_p = r'\s+INTO\s+' + _parse_identifier_p
 _parse_into_re = re.compile(_parse_into_p, re.IGNORECASE)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_insert')
+
 def _parse_insert(sql):
     return _join_identifier(_parse_into_re.search(sql))
 
-_parse_update_p = '\s*UPDATE\s+' + _parse_identifier_p
+
+_parse_update_p = r'\s*UPDATE\s+' + _parse_identifier_p
 _parse_update_re = re.compile(_parse_update_p, re.IGNORECASE)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_update')
+
 def _parse_update(sql):
     return _join_identifier(_parse_update_re.search(sql))
 
-_parse_table_p = '\s+TABLE\s+' + _parse_identifier_p
+
+_parse_table_p = r'\s+TABLE\s+' + _parse_identifier_p
 _parse_table_re = re.compile(_parse_table_p, re.IGNORECASE)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_create')
+
 def _parse_create(sql):
     return _join_identifier(_parse_table_re.search(sql))
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_drop')
+
 def _parse_drop(sql):
     return _join_identifier(_parse_table_re.search(sql))
 
+
+_parse_call_p = r'\s*CALL\s+(?!\()(\w+(\.\w+)*)'
+_parse_call_re = re.compile(_parse_call_p, re.IGNORECASE)
+
+
+def _parse_call(sql):
+    return _parse_default(sql, _parse_call_re)
+
 # TODO Following need to be reviewed again. They aren't currently used
 # in actual use as only parse out target for select/insert/update/delete.
 
-_parse_call_p = r'\s*CALL\s+(?!\()(\w+)'
-_parse_call_re = re.compile(_parse_call_p, re.IGNORECASE)
-
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_call')
-def _parse_call(sql):
-    return _parse_default(sql, _parse_call_re)
 
 _parse_show_p = r'\s*SHOW\s+(.*)'
 _parse_show_re = re.compile(_parse_show_p, re.IGNORECASE | re.DOTALL)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_show')
+
 def _parse_show(sql):
     return _parse_default(sql, _parse_show_re)
 
+
 _parse_set_p = r'\s*SET\s+(.*?)\W+.*'
 _parse_set_re = re.compile(_parse_set_p, re.IGNORECASE | re.DOTALL)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_set')
+
 def _parse_set(sql):
     return _parse_default(sql, _parse_set_re)
 
+
 _parse_exec_p = r'\s*EXEC\s+(?!\()(\w+)'
 _parse_exec_re = re.compile(_parse_exec_p, re.IGNORECASE)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_exec')
+
 def _parse_exec(sql):
     return _parse_default(sql, _parse_exec_re)
 
+
 _parse_execute_p = r'\s*EXECUTE\s+(?!\()(\w+)'
 _parse_execute_re = re.compile(_parse_execute_p, re.IGNORECASE)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_execute')
+
 def _parse_execute(sql):
     return _parse_default(sql, _parse_execute_re)
 
+
 _parse_alter_p = r'\s*ALTER\s+(?!\()(\w+)'
 _parse_alter_re = re.compile(_parse_alter_p, re.IGNORECASE)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target_alter')
+
 def _parse_alter(sql):
     return _parse_default(sql, _parse_alter_re)
 
+# For SQL queries, if a target of some sort, such as a table can be
+# meaningfully extracted, then this table should map to the function
+# which extracts it. If no target can be extracted, but it is still
+# desired that the operation be broken out separately with new Datastore
+# metrics, then the operation should still be added, but with the value
+# being set to None.
+
+
 _operation_table = {
     'select': _parse_select,
     'delete': _parse_delete,
     'insert': _parse_insert,
     'update': _parse_update,
-    'create': _parse_create,
-    'drop': _parse_drop,
+    'create': None,
+    'drop': None,
     'call': _parse_call,
-    'show': _parse_show,
-    'set': _parse_set,
-    'exec': _parse_exec,
-    'execute': _parse_execute,
-    'alter': _parse_alter,
+    'show': None,
+    'set': None,
+    'exec': None,
+    'execute': None,
+    'alter': None,
+    'commit': None,
+    'rollback': None,
 }
 
 _parse_operation_p = r'(\w+)'
 _parse_operation_re = re.compile(_parse_operation_p)
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_operation')
+
 def _parse_operation(sql):
     match = _parse_operation_re.search(sql)
     operation = match and match.group(1).lower() or ''
     return operation if operation in _operation_table else ''
 
-@internal_trace('Supportability/DatabaseUtils/Calls/parse_target')
+
 def _parse_target(sql, operation):
+    sql = sql.rstrip(';')
     parse = _operation_table.get(operation, None)
     return parse and parse(sql) or ''
 
@@ -349,6 +425,7 @@ def _parse_target(sql, operation):
 # want to match anything we want to keep. Finally match all remaining
 # numeric values. These we will also swap with a token value.
 
+
 _explain_plan_postgresql_re_1_mask_false = re.compile(
     r"""((?P<double_quotes>"[^"]*")|"""
     r"""(?P<single_quotes>'([^']|'')*')|"""
@@ -365,6 +442,7 @@ _explain_plan_postgresql_re_1_mask_true = re.compile(
 _explain_plan_postgresql_re_2 = re.compile(
     r"""^(?P<label>[^:]*:\s+).*$""", re.MULTILINE)
 
+
 def _obfuscate_explain_plan_postgresql_substitute(text, mask):
     # Perform substitutions for the explain plan on the text string.
 
@@ -387,6 +465,7 @@ def _obfuscate_explain_plan_postgresql_substitute(text, mask):
     else:
         return _explain_plan_postgresql_re_1_mask_false.sub(replacement, text)
 
+
 def _obfuscate_explain_plan_postgresql(columns, rows, mask=None):
     settings = global_settings()
 
@@ -415,11 +494,11 @@ def _obfuscate_explain_plan_postgresql(columns, rows, mask=None):
 
     text = _obfuscate_explain_plan_postgresql_substitute(text, mask)
 
-    # The mask option dictates whether we use the slightly more aggresive
+    # The mask option dictates whether we use the slightly more aggressive
     # obfuscation and simply mask out any line preceded by a label.
 
     if mask:
-        text = _explain_plan_postgresql_re_2.sub('\g<label>?', text)
+        text = _explain_plan_postgresql_re_2.sub(r'\g<label>?', text)
 
     # Now regenerate the list of rows by splitting again on newline.
 
@@ -427,16 +506,19 @@ def _obfuscate_explain_plan_postgresql(columns, rows, mask=None):
 
     return columns, rows
 
+
 _obfuscate_explain_plan_table = {
-    'PostgreSQL': _obfuscate_explain_plan_postgresql
+    'Postgres': _obfuscate_explain_plan_postgresql
 }
 
+
 def _obfuscate_explain_plan(database, columns, rows):
-    obfuscator = _obfuscate_explain_plan_table.get(database.name)
+    obfuscator = _obfuscate_explain_plan_table.get(database.product)
     if obfuscator:
         return obfuscator(columns, rows)
     return columns, rows
 
+
 class SQLConnection(object):
 
     def __init__(self, database, connection):
@@ -476,6 +558,7 @@ class SQLConnection(object):
 
         self.connection.close()
 
+
 class SQLConnections(object):
 
     def __init__(self, maximum=4):
@@ -514,7 +597,7 @@ class SQLConnections(object):
             if len(self.connections) == self.maximum:
                 connection = self.connections.pop(0)[1]
 
-                internal_metric('Supportability/DatabaseUtils/Counts/'
+                internal_metric('Supportability/Python/DatabaseUtils/Counts/'
                                 'drop_database_connection', 1)
 
                 if settings.debug.log_explain_plan_queries:
@@ -529,9 +612,6 @@ class SQLConnections(object):
 
             self.connections.append((key, connection))
 
-            internal_metric('Supportability/DatabaseUtils/Counts/'
-                            'create_database_connection', 1)
-
             if settings.debug.log_explain_plan_queries:
                 _logger.debug('Created database connection for %r.',
                         database.client)
@@ -555,6 +635,7 @@ class SQLConnections(object):
     def __exit__(self, exc, value, tb):
         self.cleanup()
 
+
 def _query_result_dicts_to_tuples(columns, rows):
     # Query results come back as a list of rows. If each row is a
     # dict, then its keys can be found in the columns list. Here, we
@@ -568,14 +649,27 @@ def _query_result_dicts_to_tuples(columns, rows):
 
     return [tuple([row[col] for col in columns]) for row in rows]
 
-@internal_trace('Supportability/DatabaseUtils/Calls/explain_plan')
+
+def _could_be_multi_query(sql):
+    return sql.rstrip().rstrip(';').count(';') > 0
+
+
 def _explain_plan(connections, sql, database, connect_params, cursor_params,
         sql_parameters, execute_params):
 
-    query = '%s %s' % (database.explain_query, sql)
-
     settings = global_settings()
 
+    if _could_be_multi_query(sql):
+        if settings.debug.log_explain_plan_queries:
+            _logger.debug('Skipping explain plan for %r on %r due to '
+                    'semicolons in the query string.', sql, database.client)
+        else:
+            _logger.debug('Skipping explain plan on %s due to '
+                    'semicolons in the query string.', database.client)
+        return None
+
+    query = '%s %s' % (database.explain_query, sql)
+
     if settings.debug.log_explain_plan_queries:
         _logger.debug('Executing explain plan for %r on %r.', query,
                 database.client)
@@ -641,6 +735,7 @@ def _explain_plan(connections, sql, database, connect_params, cursor_params,
 
     return None
 
+
 def explain_plan(connections, sql_statement, connect_params, cursor_params,
         sql_parameters, execute_params, sql_format):
 
@@ -667,22 +762,8 @@ def explain_plan(connections, sql_statement, connect_params, cursor_params,
 
     return details
 
-# Wrapper for information about a specific database. We haven't yet
-# added high level instrumentation modules for all databases we know
-# of and so we still maintain a table here indexed by Python module
-# name for some.
+# Wrapper for information about a specific database.
 
-DATABASE_MODULES = {
-    'ibm_db_dbi': 'DB2'
-}
-
-DATABASE_DEFINTIONS = {
-    'DB2': {
-        'quoting_style': 'single',
-        'explain_query': 'EXPLAIN',
-        'explain_stmts': ('select', 'insert', 'update', 'delete')
-    }
-}
 
 class SQLDatabase(object):
 
@@ -693,16 +774,8 @@ class SQLDatabase(object):
         return getattr(self.dbapi2_module, name)
 
     @property
-    def name(self):
-        name = getattr(self.dbapi2_module, '_nr_database_name', None)
-
-        if name is None:
-            name = getattr(self.dbapi2_module, '__name__', None)
-
-            if name:
-                name = DATABASE_MODULES.get(name)
-
-        return name
+    def product(self):
+        return getattr(self.dbapi2_module, '_nr_database_product', None)
 
     @property
     def client(self):
@@ -718,44 +791,27 @@ class SQLDatabase(object):
         result = getattr(self.dbapi2_module, '_nr_quoting_style', None)
 
         if result is None:
-            details = DATABASE_DEFINTIONS.get(self.name)
-            if details:
-                result = details['quoting_style']
-            else:
-                result = 'single'
+            result = 'single'
 
         return result
 
     @property
     def explain_query(self):
-        result = getattr(self.dbapi2_module, '_nr_explain_query', None)
-
-        if result is None:
-            details = DATABASE_DEFINTIONS.get(self.name)
-            if details:
-                result = details['explain_query']
-
-        return result
+        return getattr(self.dbapi2_module, '_nr_explain_query', None)
 
     @property
     def explain_stmts(self):
         result = getattr(self.dbapi2_module, '_nr_explain_stmts', None)
 
         if result is None:
-            details = DATABASE_DEFINTIONS.get(self.name)
-            if details:
-                result = details['explain_stmts']
-            else:
-                result = ()
+            result = ()
 
         return result
 
+
 class SQLStatement(object):
 
     def __init__(self, sql, database=None):
-        self.sql = sql
-        self.database = database
-
         self._operation = None
         self._target = None
         self._uncommented = None
@@ -763,6 +819,24 @@ class SQLStatement(object):
         self._normalized = None
         self._identifier = None
 
+        if isinstance(sql, six.binary_type):
+            try:
+                sql = sql.decode('utf-8')
+            except UnicodeError as e:
+                settings = global_settings()
+                if settings.debug.log_explain_plan_queries:
+                    _logger.debug('An error occurred while decoding sql '
+                            'statement: %s' % e.reason)
+
+                self._operation = ''
+                self._target = ''
+                self._uncommented = ''
+                self._obfuscated = ''
+                self._normalized = ''
+
+        self.sql = sql
+        self.database = database
+
     @property
     def operation(self):
         if self._operation is None:
@@ -784,7 +858,8 @@ class SQLStatement(object):
     @property
     def obfuscated(self):
         if self._obfuscated is None:
-            self._obfuscated = _obfuscate_sql(self.uncommented, self.database)
+            self._obfuscated = _uncomment_sql(_obfuscate_sql(self.sql,
+                self.database))
         return self._obfuscated
 
     @property
@@ -809,8 +884,10 @@ class SQLStatement(object):
         else:
             return self.obfuscated
 
+
 _sql_statements = weakref.WeakValueDictionary()
 
+
 def sql_statement(sql, dbapi2_module):
     key = (sql, dbapi2_module)
 
