@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """This module provides functions to collect information about the operating
 system, Python and hosting environment.
 
@@ -5,23 +19,24 @@ system, Python and hosting environment.
 
 import newrelic
 
-from ..common.system_info import (total_physical_memory,
+from newrelic.common.system_info import (total_physical_memory,
     logical_processor_count, physical_processor_count)
 
 import sys
 import os
 import platform
 
-try:
-    import pkg_resources
-except ImportError:
-    pass
+# try:
+#     import pkg_resources
+# except ImportError:
+#     pass
 
 try:
     import newrelic.core._thread_utilization
 except ImportError:
     pass
 
+
 def environment_settings():
     """Returns an array of arrays of environment settings
 
@@ -33,10 +48,6 @@ def environment_settings():
 
     env.append(('Agent Version', '.'.join(map(str, newrelic.version_info))))
 
-    if 'NEW_RELIC_ADMIN_COMMAND' in os.environ:
-        env.append(('Admin Command', os.environ['NEW_RELIC_ADMIN_COMMAND']))
-        del os.environ['NEW_RELIC_ADMIN_COMMAND']
-
     # System information.
 
     env.append(('Arch', platform.machine()))
@@ -137,13 +148,6 @@ def environment_settings():
     if not dispatcher and 'flup.server.cgi' in sys.modules:
         dispatcher.append(('Dispatcher', 'flup/cgi'))
 
-    if not dispatcher and 'tornado' in sys.modules:
-        dispatcher.append(('Dispatcher', 'tornado'))
-        tornado = sys.modules['tornado']
-        if hasattr(tornado, 'version_info'):
-            dispatcher.append(('Dispatcher Version',
-                               str(tornado.version_info)))
-
     if not dispatcher and 'gunicorn' in sys.modules:
         if 'gunicorn.workers.ggevent' in sys.modules:
             dispatcher.append(('Dispatcher', 'gunicorn (gevent)'))
@@ -155,6 +159,13 @@ def environment_settings():
         if hasattr(gunicorn, '__version__'):
             dispatcher.append(('Dispatcher Version', gunicorn.__version__))
 
+    if not dispatcher and 'tornado' in sys.modules:
+        dispatcher.append(('Dispatcher', 'tornado'))
+        tornado = sys.modules['tornado']
+        if hasattr(tornado, 'version_info'):
+            dispatcher.append(('Dispatcher Version',
+                               str(tornado.version_info)))
+
     env.extend(dispatcher)
 
     # Module information.
@@ -171,17 +182,29 @@ def environment_settings():
     # TL;DR: Do NOT use six module for the following iteration.
 
     for name, module in list(sys.modules.items()):
+        # If the module isn't actually loaded (such as failed relative imports
+        # in Python 2.7), the module will be None and should not be reported.
+        if module is None:
+            continue
+
         if name.startswith('newrelic.hooks.'):
             plugins.append(name)
 
         elif name.find('.') == -1 and hasattr(module, '__file__'):
-            try:
-                if 'pkg_resources' in sys.modules:
-                    version = pkg_resources.get_distribution(name).version
-                    if version:
-                        name = '%s (%s)' % (name, version)
-            except Exception:
-                pass
+            # XXX This is disabled as it can cause notable overhead in
+            # pathalogical cases. Will be replaced with a new system
+            # where have a whitelist of packages we really want version
+            # information for and will work out on case by case basis
+            # how to extract that from the modules themselves.
+
+            # try:
+            #     if 'pkg_resources' in sys.modules:
+            #         version = pkg_resources.get_distribution(name).version
+            #         if version:
+            #             name = '%s (%s)' % (name, version)
+            # except Exception:
+            #     pass
+
             plugins.append(name)
 
     env.append(('Plugin List', plugins))
