@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """Instrumentation module for CherryPy framework.
 
 """
@@ -20,25 +34,38 @@
 # * We don't track time spent in hook functions which may be registered
 #   for events such as before_handler, on_end_request etc.
 #
-# * We don't handle any sub dispatching that may be occuring due to the
+# * We don't handle any sub dispatching that may be occurring due to the
 #   use of XMLRPCDispatcher.
 
+from newrelic.api.function_trace import FunctionTrace, wrap_function_trace
+from newrelic.api.transaction import current_transaction
+from newrelic.api.time_trace import record_exception
+from newrelic.api.wsgi_application import wrap_wsgi_application
+from newrelic.common.object_names import callable_name
+from newrelic.common.object_wrapper import (ObjectProxy, function_wrapper,
+        wrap_function_wrapper)
+from newrelic.core.config import ignore_status_code
+from newrelic.api.error_trace import wrap_error_trace
 
-from newrelic.agent import (current_transaction, wrap_wsgi_application,
-        FunctionTrace, callable_name, ObjectProxy, function_wrapper,
-        wrap_function_wrapper, wrap_function_trace, ignore_status_code)
 
 def framework_details():
     import cherrypy
     return ('CherryPy', getattr(cherrypy, '__version__', None))
 
+
 def should_ignore(exc, value, tb):
     from cherrypy import HTTPError, HTTPRedirect
 
     # Ignore certain exceptions based on HTTP status codes.
 
     if isinstance(value, (HTTPError, HTTPRedirect)):
-        if ignore_status_code(value.status):
+        # In the case of an HTTPRedirect, value will not have a code attr.
+        # In the case of an HTTPError that is malformed (e.g.
+        # HTTPError("10 Bad error")), value will not have a code attr.
+        # In both of those cases, we fall back to value.status
+        code = getattr(value, 'code', value.status)
+
+        if ignore_status_code(code):
             return True
 
     # Ignore certain exceptions based on their name.
@@ -52,6 +79,7 @@ def should_ignore(exc, value, tb):
     if fullname in ignore_exceptions:
         return True
 
+
 @function_wrapper
 def handler_wrapper(wrapped, instance, args, kwargs):
     transaction = current_transaction()
@@ -65,13 +93,9 @@ def handler_wrapper(wrapped, instance, args, kwargs):
     # except those which are used for controlling the actions of the
     # application.
 
-    with FunctionTrace(transaction, name=name):
-        try:
-            return wrapped(*args, **kwargs)
+    with FunctionTrace(name=name):
+        return wrapped(*args, **kwargs)
 
-        except:  # Catch all
-            transaction.record_exception(ignore_errors=should_ignore)
-            raise
 
 class ResourceProxy(ObjectProxy):
 
@@ -83,6 +107,7 @@ class ResourceProxy(ObjectProxy):
         attr = super(ResourceProxy, self).__getattr__(name)
         return name.isupper() and handler_wrapper(attr) or attr
 
+
 def wrapper_Dispatcher_find_handler(wrapped, instance, args, kwargs):
     transaction = current_transaction()
 
@@ -97,8 +122,7 @@ def wrapper_Dispatcher_find_handler(wrapped, instance, args, kwargs):
     except:  # Catch all
         # Can end up here when a custom _cp_dispatch() method is
         # used and that raises an exception.
-
-        transaction.record_exception()
+        record_exception()
         raise
 
     if obj:
@@ -135,6 +159,7 @@ def wrapper_Dispatcher_find_handler(wrapped, instance, args, kwargs):
 
     return obj, vpath
 
+
 def wrapper_RoutesDispatcher_find_handler(wrapped, instance, args, kwargs):
     transaction = current_transaction()
 
@@ -148,8 +173,7 @@ def wrapper_RoutesDispatcher_find_handler(wrapped, instance, args, kwargs):
 
     except:  # Catch all
         # Can end up here when the URL was invalid in some way.
-
-        transaction.record_exception()
+        record_exception()
         raise
 
     if handler:
@@ -166,23 +190,30 @@ def wrapper_RoutesDispatcher_find_handler(wrapped, instance, args, kwargs):
 
     return handler
 
+
 def instrument_cherrypy__cpreqbody(module):
     wrap_function_trace(module, 'process_multipart')
     wrap_function_trace(module, 'process_multipart_form_data')
 
+
 def instrument_cherrypy__cprequest(module):
     wrap_function_trace(module, 'Request.handle_error')
 
+
 def instrument_cherrypy__cpdispatch(module):
     wrap_function_wrapper(module, 'Dispatcher.find_handler',
             wrapper_Dispatcher_find_handler)
     wrap_function_wrapper(module, 'RoutesDispatcher.find_handler',
             wrapper_RoutesDispatcher_find_handler)
+    wrap_error_trace(module, 'PageHandler.__call__',
+            ignore_errors=should_ignore)
+
 
 def instrument_cherrypy__cpwsgi(module):
     wrap_wsgi_application(module, 'CPWSGIApp.__call__',
             framework=framework_details())
 
+
 def instrument_cherrypy__cptree(module):
     wrap_wsgi_application(module, 'Application.__call__',
             framework=framework_details())
